#!/bin/ash

# Get HTTP ACTION
FILEPATH="`/opt/webserver/cgi-bin/getarg path`"
DELETE="`/opt/webserver/cgi-bin/getarg del`"
FOLDER="`/opt/webserver/cgi-bin/getarg folder`"

#Default is root directory
if [[ "x$FILEPATH" = "x" -o "$FILEPATH" = "/" ]]; then
  FILEPATH="/mnt/debug/Stations"
  DIR=""
else
  DIR="`echo $FILEPATH | sed 's,/$,,'`"
  FILEPATH="/mnt/debug/Stations$FILEPATH"
fi

# Work out what we are looking at (can't cd into a file)
cd "$FILEPATH"
if [[ "$FILEPATH" = "`pwd`" -o "$FILEPATH" = "`pwd`/" ]]; then
  type="dir"
else
  type="file"
fi

# Handle File removal
# Restrict to files within the stations database area
if [ "$DELETE" = "y" ]; then
  echo -e "Content-Type text/html\r\n\r"
  echo "<html><head><title>Manage stations</title>"
  echo "<link rel='STYLESHEET' href='/sharpfin.css' type='text/css' />"
  echo "</head><body>"
  if [ "$type" = "file" ]; then
    rm -f "$FILEPATH"
    killall -SIGHUP ushare 
    echo "<h1>File removed</h1>"
  else
    if [ "`ls "$FILEPATH" | wc -l`" -gt 0 ]; then
      echo "<h1>Folder not empty - delete contents before removing the folder</h1>"
    else
      rmdir "$FILEPATH"
      echo "<h1>Folder deleted</h1>"
    fi
  fi
  echo "<a href=\"/cgi-bin/radio_Stations_Manage.cgi\"><span style=\"font-family:monospace\">Continue</span></a>"
  echo "</body></html>"
  exit
fi

# Handle creating a new folder
if [ "$type" = "dir" -a "$FOLDER" ]; then

  echo -e "Content-Type text/html\r\n\r"
  echo "<html><head><title>Manage stations</title>"
  echo "<link rel='STYLESHEET' href='/sharpfin.css' type='text/css' />"
  echo "</head><body><h1>Folders</h1>"

  folder="`echo $FOLDER | sed 's/[^a-zA-Z0-9 -]//g'`"
 
  if [[ -e "$FILEPATH/$folder" ]]; then
    echo "<p>Folder already exists: Stations$DIR/$folder</p>"
  else
    mkdir "$FILEPATH/$folder"
    echo "<p>New folder created</p>"
  fi

  echo "<a href=\"/cgi-bin/radio_Stations_Manage.cgi\"><span style=\"font-family:monospace\">Continue</span></a>"
  echo "</body></html>"
  exit
fi

# Handle File download
if [ "$type" = "file" ]; then
  len="`ls -l \"$FILEPATH\" | cut -c30-42 | sed -e 's/ //g'`"
  echo -e "Content-Type binary/octet-stream\r\n\r"
  echo "Content-Length $length"
  echo "Content-Disposition attachment; filename=$DIR; size=$len"
  echo ""
  cat "$FILEPATH"
  exit
fi

# Handle directory ACTION
if [ "$type" = "dir" ]; then

  echo -e "Content-Type text/html\r\n\r"
  echo "<html><head><title>Manage stations</title>"
  echo "<link rel='STYLESHEET' href='/sharpfin.css' type='text/css' />"
  echo "</head><body>"

  if [ "$REQUEST_METHOD" = "POST" ]; then
    echo "<h1>File upload</h1>"
    echo "<p>File uploading...</p>"

    if [ "$CONTENT_LENGTH" -gt 0 ]; then
      # Discard first four lines
      j = 0
      while read line; do
         if [ -z $(echo "$line"|sed "/filename/d") ]; then
            filename="`echo $line | grep -o 'filename="[^"]*"' | cut -c11-50 | sed 's/.$//' `"
         fi
         j=$(($j + 1))
         if [[ $j -gt 3 ]]; then
            break
         fi
      done

      filename="`echo $filename | sed 's/[^a-zA-Z0-9 .-]//g'`"

      if [[ -e "$FILEPATH/$filename" ]]; then
         echo "<p>Filename already exists - please remove existing file first</p>"
      else
         echo "<p>Checked filename - OK</p>"
         while read line; do
            linestart="`echo $line | cut -c1-4 `"
            if [[ $linestart = "#EXT" -o $linestart = "http" ]]; then
              echo $line | tee -a "$FILEPATH/$filename" >/dev/null
            fi
            if [[ $linestart = "----" ]]; then
              break
            fi
         done
         echo "<p>File uploaded - OK</p>"
         killall -SIGHUP ushare
      fi
    fi
  fi

  echo "<h1>Folder: Stations`echo $DIR`</h1>"

  IFS=$(echo -en "\n\b")
  ENTRIES="`ls|sed -e 's/ /\ /g'`"

  echo "<table border=0 cellspacing=0 cellpadding=4>"

  if [ "$DIR" != "" ]; then
    PARENT="`cd .. ; pwd | cut -c20-100`"
    iinfo="<a href=\"/cgi-bin/radio_Stations_Manage.cgi?path=$PARENT/\"><span style=\"font-family:monospace\">.. Up a level</span></a>"
    echo "<tr><td valign=top>$iinfo</td><td></td><td></td></tr>"
  fi

  for i in $ENTRIES; do
    if [ "$i" = 1 ]; then
       echo "<tr><td valign=top>Station list - click to download</td><td>Delete</td><td>Size information</td></tr>"
    fi
    iinfo="`/bin/ls -lad \"$i\" | cut -c1-55`"
    iencode="`echo $i | sed -e 's/ /%20/g'`"
    ilink="<a href=\"/cgi-bin/radio_Stations_Manage.cgi?path=$DIR/$iencode\"><span style=\"font-family:monospace\">$i</span></a>"
    dellink="<a href=\"/cgi-bin/radio_Stations_Manage.cgi?del=y&path=$DIR/$iencode\" onclick=\"return confirm('Deleting station list. Are you sure?')\" title=\"Delete this station list\"><span style=\"font-family:monospace;color:Red\">X</span></a>"
    echo "<tr><td valign=top>$ilink</td>"
    echo "<td valign=top>$dellink</td>"
    echo "<td valign=top><span style=\"font-family: monospace\">$iinfo</span></td></tr>"
  done
  echo "</table>"

  echo "<h1>Space available on the radio</h1>"
  echo "Space available (1 k blocks):<br> `df /opt/ | cut -c35-55 | sed -e 's/%/%<br>/g' | sed 's/ \{1,\}/~/g' ` <br>"
  echo "Space used by station lists: `du -sh /opt/Stations/ | sed -e 's/\/opt\/Stations//g' `"

  echo "<h1>Upload new stations list to the current folder</h1>"  

  echo "<p>Station lists must be in m3u format with the tags shown below and can list multiple stations.</p>" 
  echo "<p>Filenames may contain A-Z, 0-9, spaces and dashes, and must have the extension '.m3u'.<br>Allow a few seconds for files to upload to your radio.</p>"
  echo "<p><span style=\"font-family:monospace\">#EXTM3U<br>
  #EXTINF:Station 1 name<br>
  http://stream.address.com/example1<br>
  #EXTINF:Station 2 name<br>
  http://stream.address.com/moremusic</span></p>"

  echo "<form method=\"POST\" enctype=\"multipart/form-data\" action=\"/cgi-bin/radio_Stations_Manage.cgi?path=$DIR\">"
  echo "New stations list: <input name=\"stationsfile\" id=\"stationsfile\" type=\"file\" accept=\".m3u\" />"
  echo "<button type=\"submit\">Upload list</button>"
  echo "</form>"

  echo "<h1>Create new stations folder</h1>"

  echo "<p>Use folders to organise stations for easier browsing on the radio, e.g. by country or genre</p>"
  echo "<p>Folder names may contain A-Z, 0-9, spaces and dashes.</p>"

  echo "<form method=\"GET\" action=\"/cgi-bin/radio_Stations_Manage.cgi\">"
  echo "New folder: <input name=\"folder\" id=\"folder\" type=\"text\" />"
  echo "<input name=\"path\" id=\"path\" type=\"hidden\" value=\"$DIR\"/>"
  echo "<button type=\"submit\">Create folder</button>"
  echo "</form>"

  echo "</body></html>"
fi
